==> dlrcomputervision-2.1.zip: dlrcomputervision-2.1/dlrComputerVision/test/cameraIntrinsicsPinholeTest.cpp <==
/**
***************************************************************************
* @file cameraIntrinsicsPinholeTest.cpp
*
* Source file defining tests for the CameraIntrinsicsPinhole class.
*
* Copyright (C) 2006-2007 David LaRose, dlr@cs.cmu.edu
* See accompanying file, LICENSE.TXT, for details.
*
* $Revision: $
* $Date: $
***************************************************************************
**/

#include <dlrCommon/functional.h>
#include <dlrComputerVision/cameraIntrinsicsPinhole.h>
#include <dlrTest/testFixture.h>

using namespace dlr::common;
using namespace dlr::computerVision;
using namespace dlr::geometry;
using namespace dlr::numeric;
using namespace dlr::test;


class CameraIntrinsicsPinholeTest
  : public TestFixture<CameraIntrinsicsPinholeTest> {

public:

  CameraIntrinsicsPinholeTest();
  ~CameraIntrinsicsPinholeTest() {}

  void setUp(const std::string& testName) {}
  void tearDown(const std::string& testName) {}

  // Tests.
  void testConstructor__void();
  void testConstructor__args();
  void testProject();
  void testReverseProject();

private:

  double m_defaultTolerance;
  
}; // class CameraIntrinsicsPinholeTest


/* ============== Member Function Definititions ============== */

CameraIntrinsicsPinholeTest::
CameraIntrinsicsPinholeTest()
  : TestFixture<CameraIntrinsicsPinholeTest>("CameraIntrinsicsPinholeTest"),
    m_defaultTolerance(1.0E-10)    
{
  DLR_TEST_REGISTER_MEMBER(testConstructor__void);
  DLR_TEST_REGISTER_MEMBER(testConstructor__args);
  DLR_TEST_REGISTER_MEMBER(testProject);
  DLR_TEST_REGISTER_MEMBER(testReverseProject);
}


void
CameraIntrinsicsPinholeTest::
testConstructor__void()
{
  // Pass.
}


void
CameraIntrinsicsPinholeTest::
testConstructor__args()
{
  // Tested in testReverseProject().
}


void
CameraIntrinsicsPinholeTest::
testProject()
{
  // Arbitrary camera params.
  size_t numPixelsX = 320;
  size_t numPixelsY = 240;
  double focalLength = 0.03;
  double pixelSizeX = 0.001;
  double pixelSizeY = 0.002;
  double centerU = 100.0;
  double centerV = 125.0;
  CameraIntrinsicsPinhole intrinsics(numPixelsX, numPixelsY,
                                     0.03, 0.001, 0.002, 100, 125);

  for(double zCoord = 1.0; zCoord < 10.0; zCoord += 0.7) {
    for(double yCoord = -1.0; yCoord < 1.0; yCoord += 0.1) {
      for(double xCoord = -1.0; xCoord < 1.0; xCoord += 0.1) {
        Vector3D cameraCoord(xCoord, yCoord, zCoord);
        Vector2D pixelCoord = intrinsics.project(cameraCoord);
        double referenceU =
          centerU + (xCoord * focalLength) / (pixelSizeX * zCoord);
        double referenceV =
          centerV + (yCoord * focalLength) / (pixelSizeY * zCoord);
        DLR_TEST_ASSERT(
          approximatelyEqual(pixelCoord.x(), referenceU, m_defaultTolerance));
        DLR_TEST_ASSERT(
          approximatelyEqual(pixelCoord.y(), referenceV, m_defaultTolerance));
      }
    }
  }
}


void
CameraIntrinsicsPinholeTest::
testReverseProject()
{
  // We test by round trip against convertWorldPointToPixel(), which
  // has its own independent test.

  // Arbitrary camera params.
  size_t numPixelsX = 320;
  size_t numPixelsY = 240;
  CameraIntrinsicsPinhole intrinsics(numPixelsX, numPixelsY,
                                     0.03, 0.001, 0.002, 100, 125);

  for(double vCoord = 0.0; vCoord < numPixelsY; vCoord += 1.2) {
    for(double uCoord = 0.0; uCoord < numPixelsX; uCoord += 1.2) {
      Vector2D pixelCoord(uCoord, vCoord);
      Ray3D ray = intrinsics.reverseProject(pixelCoord);
      Vector3D pointOnRay = ray.getOrigin() + 12.0 * ray.getDirectionVector();
      Vector2D recoveredPixelCoord = intrinsics.project(pointOnRay);
      DLR_TEST_ASSERT(approximatelyEqual(
                        recoveredPixelCoord.x(), pixelCoord.x(),
                        m_defaultTolerance));
      DLR_TEST_ASSERT(approximatelyEqual(
                        recoveredPixelCoord.y(), pixelCoord.y(),
                        m_defaultTolerance));
    }
  }
}


#if 0

int main(int argc, char** argv)
{
  CameraIntrinsicsPinholeTest currentTest;
  bool result = currentTest.run();
  return (result ? 0 : 1);
}

#else

namespace {

  CameraIntrinsicsPinholeTest currentTest;

}

#endif

==> dlrcomputervision-2.4.zip: dlrcomputervision-2.4/dlrComputerVision/test/cameraIntrinsicsPinholeTest.cpp <==
/**
***************************************************************************
* @file cameraIntrinsicsPinholeTest.cpp
*
* Source file defining tests for the CameraIntrinsicsPinhole class.
*
* Copyright (C) 2006-2007 David LaRose, dlr@cs.cmu.edu
* See accompanying file, LICENSE.TXT, for details.
*
* $Revision: $
* $Date: $
***************************************************************************
**/

#include <dlrCommon/functional.h>
#include <dlrComputerVision/cameraIntrinsicsPinhole.h>
#include <dlrTest/testFixture.h>

using namespace dlr::common;
using namespace dlr::computerVision;
using namespace dlr::geometry;
using namespace dlr::numeric;
using namespace dlr::test;


class CameraIntrinsicsPinholeTest
  : public TestFixture<CameraIntrinsicsPinholeTest> {

public:

  CameraIntrinsicsPinholeTest();
  ~CameraIntrinsicsPinholeTest() {}

  void setUp(const std::string& testName) {}
  void tearDown(const std::string& testName) {}

  // Tests.
  void testAccessorFunctions();
  void testConstructor__void();
  void testConstructor__args();
  void testGetProjectionMatrix();
  void testProject();
  void testReverseProject();
  
private:

  double m_defaultTolerance;
  double m_relaxedTolerance;
  
}; // class CameraIntrinsicsPinholeTest


/* ============== Member Function Definititions ============== */

CameraIntrinsicsPinholeTest::
CameraIntrinsicsPinholeTest()
  : TestFixture<CameraIntrinsicsPinholeTest>("CameraIntrinsicsPinholeTest"),
    m_defaultTolerance(1.0E-10),
    m_relaxedTolerance(1.0E-7)
{
  DLR_TEST_REGISTER_MEMBER(testAccessorFunctions);
  DLR_TEST_REGISTER_MEMBER(testConstructor__void);
  DLR_TEST_REGISTER_MEMBER(testConstructor__args);
  DLR_TEST_REGISTER_MEMBER(testGetProjectionMatrix);
  DLR_TEST_REGISTER_MEMBER(testProject);
  DLR_TEST_REGISTER_MEMBER(testReverseProject);
}


void
CameraIntrinsicsPinholeTest::
testAccessorFunctions()
{
  // Arbitrary camera params.
  size_t numPixelsX = 320;
  size_t numPixelsY = 240;
  double focalLength = 0.03;
  double pixelSizeX = 0.001;
  double pixelSizeY = 0.002;
  double centerU = 100.0;
  double centerV = 125.0;
  CameraIntrinsicsPinhole intrinsics(numPixelsX, numPixelsY,
                                     0.03, 0.001, 0.002, 100, 125);

  DLR_TEST_ASSERT(intrinsics.getNumPixelsX() == numPixelsX);
  DLR_TEST_ASSERT(intrinsics.getNumPixelsY() == numPixelsY);
  DLR_TEST_ASSERT(
    approximatelyEqual(intrinsics.getFocalLength(), focalLength,
                       m_defaultTolerance));
  DLR_TEST_ASSERT(
    approximatelyEqual(intrinsics.getPixelSizeX(), pixelSizeX,
                       m_defaultTolerance));
  DLR_TEST_ASSERT(
    approximatelyEqual(intrinsics.getPixelSizeY(), pixelSizeY,
                       m_defaultTolerance));
  DLR_TEST_ASSERT(
    approximatelyEqual(intrinsics.getCenterU(), centerU,
                       m_defaultTolerance));
  DLR_TEST_ASSERT(
    approximatelyEqual(intrinsics.getCenterV(), centerV,
                       m_defaultTolerance));
}


void
CameraIntrinsicsPinholeTest::
testConstructor__void()
{
  // Pass.
}


void
CameraIntrinsicsPinholeTest::
testConstructor__args()
{
  // Tested in testReverseProject().
}


void
CameraIntrinsicsPinholeTest::
testGetProjectionMatrix()
{
  // Arbitrary camera params.
  size_t numPixelsX = 320;
  size_t numPixelsY = 240;
  double focalLength = 0.03;
  double pixelSizeX = 0.001;
  double pixelSizeY = 0.002;
  double centerU = 100.0;
  double centerV = 125.0;
  CameraIntrinsicsPinhole intrinsics(numPixelsX, numPixelsY,
                                     0.03, 0.001, 0.002, 100, 125);
  Array2D<double> projectionMatrix = intrinsics.getProjectionMatrix();

  for(double zCoord = 1.0; zCoord < 10.0; zCoord += 0.7) {
    for(double yCoord = -1.0; yCoord < 1.0; yCoord += 0.1) {
      for(double xCoord = -1.0; xCoord < 1.0; xCoord += 0.1) {
        Vector3D cameraCoord(xCoord, yCoord, zCoord);
        Vector2D pixelCoord(
          (projectionMatrix(0, 0) * cameraCoord.x()
           + projectionMatrix(0, 1) * cameraCoord.y()
           + projectionMatrix(0, 2) * cameraCoord.z()
           + projectionMatrix(0, 3)),
          (projectionMatrix(1, 0) * cameraCoord.x()
           + projectionMatrix(1, 1) * cameraCoord.y()
           + projectionMatrix(1, 2) * cameraCoord.z()
           + projectionMatrix(1, 3)),
          (projectionMatrix(2, 0) * cameraCoord.x()
           + projectionMatrix(2, 1) * cameraCoord.y()
           + projectionMatrix(2, 2) * cameraCoord.z()
           + projectionMatrix(2, 3)));
        double referenceU =
          centerU + (xCoord * focalLength) / (pixelSizeX * zCoord);
        double referenceV =
          centerV + (yCoord * focalLength) / (pixelSizeY * zCoord);
        DLR_TEST_ASSERT(
          approximatelyEqual(pixelCoord.x(), referenceU, m_defaultTolerance));
        DLR_TEST_ASSERT(
          approximatelyEqual(pixelCoord.y(), referenceV, m_defaultTolerance));
      }
    }
  }
}


void
CameraIntrinsicsPinholeTest::
testProject()
{
  // Arbitrary camera params.
  size_t numPixelsX = 320;
  size_t numPixelsY = 240;
  double focalLength = 0.03;
  double pixelSizeX = 0.001;
  double pixelSizeY = 0.002;
  double centerU = 100.0;
  double centerV = 125.0;
  CameraIntrinsicsPinhole intrinsics(
    numPixelsX, numPixelsY, focalLength, pixelSizeX, pixelSizeY,
    centerU, centerV);

  for(double zCoord = 1.0; zCoord < 10.0; zCoord += 0.7) {
    for(double yCoord = -1.0; yCoord < 1.0; yCoord += 0.1) {
      for(double xCoord = -1.0; xCoord < 1.0; xCoord += 0.1) {
        Vector3D cameraCoord(xCoord, yCoord, zCoord);
        Vector2D pixelCoord = intrinsics.project(cameraCoord);
        double referenceU =
          centerU + (xCoord * focalLength) / (pixelSizeX * zCoord);
        double referenceV =
          centerV + (yCoord * focalLength) / (pixelSizeY * zCoord);
        DLR_TEST_ASSERT(
          approximatelyEqual(pixelCoord.x(), referenceU, m_defaultTolerance));
        DLR_TEST_ASSERT(
          approximatelyEqual(pixelCoord.y(), referenceV, m_defaultTolerance));
      }
    }
  }
}


void
CameraIntrinsicsPinholeTest::
testReverseProject()
{
  // We test by round trip against convertWorldPointToPixel(), which
  // has its own independent test.

  // Arbitrary camera params.
  size_t numPixelsX = 320;
  size_t numPixelsY = 240;
  CameraIntrinsicsPinhole intrinsics(numPixelsX, numPixelsY,
                                     0.03, 0.001, 0.002, 100, 125);

  for(double vCoord = 0.0; vCoord < numPixelsY; vCoord += 1.2) {
    for(double uCoord = 0.0; uCoord < numPixelsX; uCoord += 1.2) {
      Vector2D pixelCoord(uCoord, vCoord);
      Ray3D ray = intrinsics.reverseProject(pixelCoord);
      Vector3D pointOnRay = ray.getOrigin() + 12.0 * ray.getDirectionVector();
      Vector2D recoveredPixelCoord = intrinsics.project(pointOnRay);
      DLR_TEST_ASSERT(approximatelyEqual(
                        recoveredPixelCoord.x(), pixelCoord.x(),
                        m_defaultTolerance));
      DLR_TEST_ASSERT(approximatelyEqual(
                        recoveredPixelCoord.y(), pixelCoord.y(),
                        m_defaultTolerance));
    }
  }
}


#if 0

int main(int argc, char** argv)
{
  CameraIntrinsicsPinholeTest currentTest;
  bool result = currentTest.run();
  return (result ? 0 : 1);
}

#else

namespace {

  CameraIntrinsicsPinholeTest currentTest;

}

#endif
